
/***************************************************************************
 *
 * IPAT - Interferometer Threshold Curve Plotter
 *
 * Stephen R. Whiteley (stevew@srware.com)
 * 
 * Copyright (c) 2002 Whiteley Research Inc.  All Rights Reserved. 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************
 * $Id: screen.h,v 2.8 2009/01/11 21:07:43 stevew Exp $
 ***************************************************************************/

#ifndef PI
#define PI 3.1415926535898
#endif

// two pi over phi0 in normalized units
#define PHI 3.038393

// maximum number of iterations in three jj Newton search
#define NITER 8

// Pixel map and colors
//
struct clr
{
    long pixel;
    unsigned char red;
    unsigned char green;
    unsigned char blue;
};

// linked list of x-y file points to plot
//
struct extra {
    extra() { x = y = 0; pf = 0; size = 0; next = 0; }

    float *x;
    float *y;
    int *pf;
    int size;
    struct extra *next;
};

// Drawing areas
//
struct screen
{
    screen();
    virtual ~screen() { }

    // generic drawing functions
    virtual void clear();
    virtual void define_color(int, int, int, int);
    virtual void set_color(int);
    virtual void pixel(int, int);
    virtual void line(int, int, int, int);
    virtual void box(int, int, int, int);
    virtual void text(const char*, int, int, int);
    virtual void set_linestyle(int);
    virtual void swap_pm(bool);
    virtual void update(int, int, int, int);

    // application functions
    void displ();
    void twojj(void);
    void threejj(void);
    bool solve12(double,double*);
    bool solve13(double,double*);
    bool solve32(double,double*);
    bool solve31(double,double*);
    bool solve21(double,double*);
    bool solve23(double,double*);
    void plot(double, double);
    void plotphi(double, double, double, double, double);
    void diagram();
    void show_l_icon(int, int);
    void show_x_icon(int, int);
    void xy_pick();
    void marker(int, int);
    void refmark();
    void plpts();

    // "viewport" parameters
    int X;      // scale width
    int Y;      // scale height
    int W;      // font width
    int H;      // font height
    int W2;     // half font width
    int H2;     // half font height
    int H3;     // vertical space and a half
    int XL;     // left margin
    int XU;     // right   "
    int YL;     // lower   "
    int YU;     // upper   "
    int XS;     // screen width
    int YS;     // screen height
    int XC;     // X center of plot area
    int YC;     // Y center of plot area

    // "window" parameters
    struct
    {
        double xmin;
        double xmax;
        double ymin;
        double ymax;
        int    xx;
        int    xo;
        int    xmn;
        int    xmx;
        int    yy;
        int    yo;
    } scale;

#ifdef WIN32_NATIVE
    HWND da;
    HDC gc;
    int curforeg;
    int curstyle;
    HFONT font;
#else
    GtkWidget *da;
    GdkGC *gc;
    GdkGC *xorgc;
    GdkFont *font;
    GdkColormap *cmap;
    GdkPixmap *bpm;
    GdkWindow *oldwin;
    int bpm_width, bpm_height;
#endif
    int last_x, last_y;
    bool draw_ghost;
    bool first_ghost;
};

// parameter storage
//
struct values
{
    values()
    {
        la = lb = lc = ld = 0.5;
        ix = i2 = iy = 1.0;
        lfrac = 0.5;
        sign = 1;
    }

    // helper functions for three jj computation
    bool solcheck(double,double);
    bool get1(double,double,double*);
    bool get2(double,double,double*);
    bool get3(double,double,double*);

    double la;
    double lb;
    double lc;
    double ld;
    double ix;
    double i2;
    double iy;
    double lfrac;
    int sign;
};

